@@ -2,9 +2,12 @@ import ctypes.wintypes
 from functools import cached_property
 from typing import Optional
 
-from . import utils
+from . import Keycode, utils
 from .packets import PacketHookWatcher
-from .windows import InputHandler, MemoryHandler, user32
+from .windows import MemoryHandler
+
+from .constants import user32
+
 
 WIZARD_SPEED = 580
 
@@ -19,7 +22,6 @@ class Client:
 
     def __init__(self, window_handle: int):
         self.window_handle = window_handle
-        self._input = InputHandler(window_handle)
         self._memory = MemoryHandler(self.process_id)
         self.current_zone = None
 
@@ -47,11 +49,8 @@ class Client:
         user32.GetWindowThreadProcessId(self.window_handle, ctypes.byref(pid))
         return pid.value
 
-    async def send_key(self, key: str, seconds: float):
-        await self._input.send_key(key, seconds)
-
-    # async def click(self, x: int, y: int):
-    #     await self._input.click(x, y)
+    async def send_key(self, key: Keycode, seconds: float):
+        await utils.timed_send_key(self.window_handle, key, seconds)
 
     def login(self, username: str, password: str):
         """
@@ -61,7 +60,7 @@ class Client:
             username: The username to login with
             password: The password to login with
         """
-        utils.wiz_login(self.window_handle, username, password)
+        utils.instance_login(self.window_handle, username, password)
 
     def watch_packets(self):
         """
@@ -136,19 +135,19 @@ class Client:
         yaw = utils.calculate_perfect_yaw(current_xyz, target_xyz)
 
         await self.set_yaw(yaw)
-        await self._input.send_key("W", move_seconds)
+        await utils.timed_send_key(self.window_handle, Keycode.W, move_seconds)
 
     async def teleport(
-        self, *, x: float = None, y: float = None, z: float = None, yaw: float = None
+        self, x: float = None, y: float = None, z: float = None, yaw: float = None
     ) -> bool:
         """
         Teleport the player to a set x, y, z
 
         Args:
-            x: X to teleport to
-            y: Y to teleport to
-            z: Z to teleport to
-            yaw: yaw to set
+            x: X to teleport to or None to not change
+            y: Y to teleport to or None to not change
+            z: Z to teleport to or None to not change
+            yaw: yaw to set or None to not change
 
         Raises:
             RuntimeError: player_struct hook not active
@@ -247,6 +246,30 @@ class Client:
         """
         return await self._memory.set_player_roll(pitch)
 
+    async def scale(self) -> Optional[float]:
+        """
+        Player scale
+
+        Raises:
+            RuntimeError: player_struct hook not active
+
+        Returns:
+            scale float or None if hooked function hasn't run yet
+        """
+        return await self._memory.read_player_scale()
+
+    async def set_scale(self, scale: float) -> bool:
+        """
+        Set the player scale
+
+        Raises:
+            RuntimeError: player_struct hook not active
+
+        Return:
+            True if value was set, False otherwise
+        """
+        return await self._memory.set_player_scale(scale)
+
     async def quest_xyz(self) -> Optional[utils.XYZ]:
         """
         Quest xyz
